import java.util.Vector;
import java.lang.NullPointerException;

public class MySequence<E> extends Sequence<E> {
    // Overview: Tipo delle sequenze finite di oggetti di tipo E di dimensioni e contenuto
    // modificabili i cui elementi possono essere null
    // AF = seq.get(0), ..., seq.get(dim -1)
    // IR = seq != null && 0 <= dim <= seq.size()
    
    private int dim;
    private Vector<E> seq;
    
    public MySequence () {
        dim = 0;
        seq = new Vector<E>();
    }
    
    /* restituisce la lunghezza della sequenza */
    public int length() {
        return dim;
    }
    /*
     @effects: restituisce dim
     */
    
    /* restituisce la prima posizione nella sequenza il cui elemento corrispondente e'
     uguale a elem */
    public int indexOf(E elem) {
        // if (elem == null) throw new NullPointerException();
        return seq.indexOf(elem);
    }
    /*
     @effects: restituisce seq.indexOf(elem), e dunque -1 se elem non e' presente
     */
    
    /* restituisce la prima posizione nella sequenza a partire dalla quale gli elementi
     successivi formano la sequenza s */
    public int indexOfSlide(Sequence<E> s) throws NullPointerException {
        if (s == null) throw new NullPointerException();
        for (int i = 0; i < this.length(); i++)
            if (indexOfSlide(i,s)) return i;
        return -1;
    }
    /*
     @effects: restituisce i tale che P(i) && forall m. m < i -> ! P(m), dove
     P(x) = forall k. x <= k < x+s.length() -> ((seq.get(k) == null -> s.get(k-x) == null) ||
                                                seq.get(k).equals(s.get(k-x)),
     e -1 se la sotto-sequenza s non e' presente.
     @throws: Se s == null lancia una NullPointerException (eccezione disponibile in Java, unchecked)
     */
    
    private boolean indexOfSlide(int i, Sequence<E> s) {
        for(int j = i; j < s.length(); j++) {
            if ((seq.get(j) == null) && (s.get(j-i) != null)) return false;
            if (! seq.get(j).equals(s.get(j-i))) return false;
        }
        return true;
    }
                
    /* restituisce l'elemento della sequenza in posizione i */
    public E get(int pos) throws IllegalArgumentException {
        if (pos < 0) throw new IllegalArgumentException();
        if (pos >= dim) return null;
        return seq.get(pos);
    }
    /*
     @effects: restituisce seq.get(pos) se 0 <= pos < dim oppure null se pos >= dim.
     @throws: Se pos < 0 lancia una IllegalArgumentException (eccezione disponibile in Java, unchecked)
     */

    /* modifica la sequenza inserendo alla posizione pos l'oggetto elem */
    public void update(int pos, E elem) throws IllegalArgumentException {
        // if (elem == null) throw new NullPointerException();
        if (pos < 0) throw new IllegalArgumentException();
        if (pos >= seq.size())
            resize(pos+1);
        seq.set(pos, elem);

    }
    /*
     @modify: this
     @effects: modifica la sequenza eseguendo seq.set(pos,elem), eventualmente ridimensionando seq.
     @throws: Se pos < 0 lancia una IllegalArgumentException (eccezione disponibile in Java, unchecked)
     */

    private void resize(int lim) {
        Vector<E> tmp = new Vector<E>(lim);
        for (int i = 0; i < seq.size(); i++)
            tmp.set(i,seq.get(i));
        for (int j = seq.size(); j < lim; j++)
            tmp.set(j,null);
        seq = tmp;
        dim = lim;
    }
}
